import pycosat


N = 8

def v(i, j):
    return N * i + j + 1

def to_cnf(vs, eq=False):
    if eq:
        yield vs
    for v1 in vs:
        for v2 in vs:
            if v1 < v2:
                yield [-v1, -v2]

def queens_clauses():
    # rows and columns
    for i in range(N):
        yield from to_cnf([v(i, j) for j in range(N)], True)
        yield from to_cnf([v(j, i) for j in range(N)], True)
    # diagonal
    for i in range(N - 1):
        yield from to_cnf([v(j, i + j) for j in range(N - i)])
    for i in range(1, N - 1):
        yield from to_cnf([v(j + i, j) for j in range(N - i)])
    for i in range(N - 1):
        yield from to_cnf([v(j, N - 1 - (i + j)) for j in range(N - i)])
    for i in range(1, N - 1):
        yield from to_cnf([v(j + i, N - 1 - j) for j in range(N - i)])

def solve():
    clauses = queens_clauses()
    # solve the SAT problem
    for sol in pycosat.itersolve(clauses):
        sol = set(sol)
        for i in range(N):
            print(''.join('Q' if v(i, j) in sol else '.' for j in range(N)))
        print('')

if __name__ == '__main__':
    solve()



"""
exmaple based on:

    OPIUM: Optimal Package Install/Uninstall Manager</a>
    http://www.cs.ucsd.edu/~lerner/papers/opium.pdf
"""

from pprint import pprint

import pycosat

index = {
    'a': {'depends': ['b', 'c', 'z']},
    'b': {'depends': ['d']},
    'c': {'depends': ['d|e', 'f|g']},
    'd': {'conflicts': ['e']},
    'e': {'conflicts': ['d']},
    'f': {'conflicts': ['g']},
    'g': {'conflicts': ['f']},
    'y': {'depends': ['z']},
    'z': {},
}

v = {} # map names to variable numbers
w = {} # map variable numbers to names
for i, name in enumerate(index.iterkeys()):
    v[name] = i + 1
    w[i + 1] = name

clauses = []
def to_cnf(name):
    depends = index[name].get('depends', [])
    for r in depends:
        clauses.append([-v[name]] + [v[n] for n in r.split('|')])

    conflicts = index[name].get('conflicts', [])
    for c in conflicts:
        clauses.append([-v[name], -v[c]])

for name in index.iterkeys():
    to_cnf(name)

print v
clauses.append([v['a']])

pprint([' V '.join(('-' if i<0 else '') + w[abs(i)] for i in clause)
        for clause in clauses])

for sol in pycosat.itersolve(clauses):
    print sorted(w[i] for i in sol if i > 0)



import re
import sys
from distutils.core import setup, Extension


# Read version from pycosat.c
pat = re.compile(r'#define\s+PYCOSAT_VERSION\s+"(\S+)"', re.M)
data = open('pycosat.c').read()
version = pat.search(data).group(1)


ext_kwds = dict(
    name = "pycosat",
    sources = ["pycosat.c"],
    define_macros = []
)

if '--inplace' in sys.argv:
    ext_kwds['define_macros'].append(('DONT_INCLUDE_PICOSAT', 1))
    ext_kwds['library_dirs'] = ['.']
    ext_kwds['libraries'] = ['picosat']


setup(
    name = "pycosat",
    version = version,
    author = "Ilan Schnell",
    author_email = "ilan@continuum.io",
    url = "https://github.com/ContinuumIO/pycosat",
    license = "MIT",
    classifiers = [
        "Development Status :: 6 - Mature",
        "Intended Audience :: Developers",
        "Operating System :: OS Independent",
        "Programming Language :: C",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.5",
        "Programming Language :: Python :: 2.6",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.2",
        "Programming Language :: Python :: 3.3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Topic :: Utilities",
    ],
    ext_modules = [Extension(**ext_kwds)],
    py_modules = ['test_pycosat'],
    description = "bindings to picosat (a SAT solver)",
    long_description = open('README.rst').read(),
)
